@@ -25,7 +25,8 @@ static pvstate_t pv_sig_state = NULL;
  * subsequent SIGCONT we can try writing to the terminal again, in case we
  * get backgrounded and later get foregrounded again.
  */
-static void pv_sig_ttou(int s)
+static void pv_sig_ttou( __attribute__ ((unused))
+			int s)
 {
 	int fd;
 
@@ -45,7 +46,8 @@ static void pv_sig_ttou(int s)
  * Handle SIGTSTP (stop typed at tty) by storing the time the signal
  * happened for later use by pv_sig_cont(), and then stopping the process.
  */
-static void pv_sig_tstp(int s)
+static void pv_sig_tstp( __attribute__ ((unused))
+			int s)
 {
 	gettimeofday(&(pv_sig_state->pv_sig_tstp_time), NULL);
 	raise(SIGSTOP);
@@ -57,7 +59,8 @@ static void pv_sig_tstp(int s)
  * last SIGTSTP to the elapsed time offset, and by trying to write to the
  * terminal again (by replacing the /dev/null stderr with the old stderr).
  */
-static void pv_sig_cont(int s)
+static void pv_sig_cont( __attribute__ ((unused))
+			int s)
 {
 	struct timeval tv;
 	struct termios t;
@@ -111,7 +114,8 @@ static void pv_sig_cont(int s)
 /*
  * Handle SIGWINCH (window size changed) by setting a flag.
  */
-static void pv_sig_winch(int s)
+static void pv_sig_winch( __attribute__ ((unused))
+			 int s)
 {
 	pv_sig_state->pv_sig_newsize = 1;
 }
@@ -120,7 +124,8 @@ static void pv_sig_winch(int s)
 /*
  * Handle termination signals by setting the abort flag.
  */
-static void pv_sig_term(int s)
+static void pv_sig_term( __attribute__ ((unused))
+			int s)
 {
 	pv_sig_state->pv_sig_abort = 1;
 }
@@ -211,7 +216,7 @@ void pv_sig_init(pvstate_t state)
 /*
  * Shut down signal handling.
  */
-void pv_sig_fini(pvstate_t state)
+void pv_sig_fini( __attribute__ ((unused)) pvstate_t state)
 {
 	sigaction(SIGPIPE, &(pv_sig_state->pv_sig_old_sigpipe), NULL);
 	sigaction(SIGTTOU, &(pv_sig_state->pv_sig_old_sigttou), NULL);
