@@ -52,12 +52,20 @@ int pv_watchfd_info(pvstate_t state, pvwatchfd_t info, int automatic)
 				 info->watch_pid, strerror(errno));
 		return 1;
 	}
-
+#ifdef HAVE_SNPRINTF
 	snprintf(info->file_fdinfo, sizeof(info->file_fdinfo) - 1,
-		 "/proc/%u/fdinfo/%d", info->watch_pid, info->watch_fd);
+#else
+	sprintf(info->file_fdinfo,
+#endif
+		"/proc/%u/fdinfo/%d", info->watch_pid, info->watch_fd);
+#ifdef HAVE_SNPRINTF
 	snprintf(info->file_fd, sizeof(info->file_fd) - 1,
-		 "/proc/%u/fd/%d", info->watch_pid, info->watch_fd);
+#else
+	sprintf(info->file_fd,
+#endif
+		"/proc/%u/fd/%d", info->watch_pid, info->watch_fd);
 
+	memset(info->file_fdpath, 0, sizeof(info->file_fdpath));
 	if (readlink
 	    (info->file_fd, info->file_fdpath,
 	     sizeof(info->file_fdpath) - 1) < 0) {
@@ -183,7 +191,11 @@ int pv_watchpid_scanfds(pvstate_t state, pvstate_t pristine,
 	struct pvwatchfd_s *info_array = NULL;
 	struct pvstate_s *state_array = NULL;
 
+#ifdef HAVE_SNPRINTF
 	snprintf(fd_dir, sizeof(fd_dir) - 1, "/proc/%u/fd", watch_pid);
+#else
+	sprintf(fd_dir, "/proc/%u/fd", watch_pid);
+#endif
 	dptr = opendir(fd_dir);
 	if (NULL == dptr)
 		return 1;
@@ -362,20 +374,29 @@ void pv_watchpid_setname(pvstate_t state, pvwatchfd_t info)
 
 	max_display_length = (state->width / 2) - 6;
 	if (max_display_length >= path_length) {
+#ifdef HAVE_SNPRINTF
 		snprintf(info->display_name,
-			 sizeof(info->display_name) - 1, "%4d:%s",
-			 info->watch_fd, info->file_fdpath);
+			 sizeof(info->display_name) - 1,
+#else
+		sprintf(info->display_name,
+#endif
+			"%4d:%.498s", info->watch_fd, info->file_fdpath);
 	} else {
 		int prefix_length, suffix_length;
 
 		prefix_length = max_display_length / 4;
 		suffix_length = max_display_length - prefix_length - 3;
 
+#ifdef HAVE_SNPRINTF
 		snprintf(info->display_name,
-			 sizeof(info->display_name) - 1, "%4d:%.*s...%.*s",
-			 info->watch_fd, prefix_length, info->file_fdpath,
-			 suffix_length,
-			 info->file_fdpath + path_length - suffix_length);
+			 sizeof(info->display_name) - 1,
+#else
+		sprintf(info->display_name,
+#endif
+			"%4d:%.*s...%.*s",
+			info->watch_fd, prefix_length, info->file_fdpath,
+			suffix_length,
+			info->file_fdpath + path_length - suffix_length);
 	}
 
 	debug("%s: %d: [%s]", "set name for fd", info->watch_fd,
